# _*_ coding:utf-8 _*_
"""
常用SQL类集合
\n 目前支持的有：
\n SQL(O_conn/M_conn/set_val/upd_del/clear/close) (SQL类，依赖cx_Oracle/pymysql)
\n Es(read_es) （ES类，依赖elasticsearch）
\n Mysql_db class类，依赖pymysql；(子方法：sel_dict/sel_sql/iud_sql/close/page_mus)
\n @author: 'TangKaiYue'
"""
from .t_jde import null


# SQL数据相关集合
class SQL:
    """
    【SQL数据相关集合】\n
    \n 目前已支持方法：
    \n 1、O_conn | Oracle数据库连接配置(依赖cx_Oracle)
    \n 2、M_conn | Mysql数据库连接配置(依赖pymysql)
    \n 3、set_val | 查询数据表的所有数据/第一条数据
    \n 4、upd_del | 修改/删除/新增数据表的数据
    \n 5、clear | 清空数据表
    \n 6、close | 关闭数据库连接
    """

    # Oracle数据库连接配置
    def O_conn(self: str = 'null', pwd: str = 'null', url: str = 'null'):
        """
        【Oracle数据库连接配置】（依赖cx_Oracle）\n
        :param self: str 用户名；
        :param pwd: str 密码；
        :param url: str 数据库连接(含服务名)；
        :return: 输出数据库连接结果；
        """
        try:
            # 判断必要入参
            if null(self) == 'N' or null(pwd) == 'N' or null(url) == 'N':
                return print("入参异常：连接数据库的 账号、密码、URL 都不能为空...")
            else:
                import cx_Oracle  # 模块引用
                cs = cx_Oracle.connect(str(self), str(pwd), str(url), encoding="UTF-8")
                return cs
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # Mysql数据库连接配置
    def M_conn(self: str = 'null', code: int = '3306', name: str = 'null', pwd: str = 'null', db: str = 'null'):
        """
        【Mysql数据库连接配置】（依赖pymysql）\n
        :param self: str 连接地址/IP；
        :param code: int 端口号，默认3306；
        :param name: str 用户名；
        :param pwd: str 密码；
        :param db: str  数据库名称；
        :return: 输出数据库连接结果；
        """
        try:
            # 判断必要入参
            if null(self) == 'N' or null(name) == 'N' or null(pwd) == 'N' or null(db) == 'N':
                return print("入参异常：连接数据库 地址、用户、密码、库名 都不能为空...")
            import pymysql
            conn = pymysql.connect(host=str(self), port=int(code), user=str(name), passwd=str(pwd), db=str(db),
                                   charset='utf8')
            return conn
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 查询数据表的数据(所有/第一条)
    def set_val(self: str = 'null', conn: str = 'null', tp: str = 'all'):
        """
        【查询数据表的数据(所有/第一条)】\n
        使用注意：大量数据查询不建议使用此项功能；\n
        :param self: str sql语句；
        :param conn: str 数据库连接,如：SQL.O_conn()；
        :param tp: str 查询所有/第一条(all/one),默认查全部；
        :return:  输出结果；
        """
        try:
            # 判断必要入参
            if null(conn) == 'N' or null(self) == 'N':
                return "异常：SQL语句、数据库连接值 不能为空！！！（PS:连接值可参考连接方法SQL.O_conn()）"
            # 获取游标
            with conn.cursor() as cursor:
                try:
                    cursor.execute(self)
                except Exception as e:
                    cursor.close()  # 关闭游标
                    return print('发生异常:%s \n传入的SQL：%s; 数据连接：%s' % (e, self, conn))
                if tp == 'all' or tp == 'ALL' or tp == 'All':
                    val = cursor.fetchall()
                    v_list = list(val)
                    return v_list
                if tp == 'one' or tp == 'ONE' or tp == 'One':
                    val = cursor.fetchone()
                    v_list = list(val)
                    return v_list
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 修改/删除/新增数据表的数据
    def upd_del(self: str = 'null', conn: str = 'null'):
        """
        【修改/删除/新增数据表的数据】\n
        :param self: str sql语句；
        :param conn: str 数据库连接,如：SQL.O_conn()；
        :return:  输出结果；
        """
        try:
            # 判断必要入参
            if null(conn) == 'N' or null(self) == 'N':
                return "异常：SQL语句、数据库连接值 都不能为空！！！（PS:连接值可参考连接方法SQL.O_conn()）"
            # 获取游标
            with conn.cursor() as cursor:
                try:
                    cursor.execute(self)  # 执行SQL
                except Exception as err:
                    conn.rollback()  # 事务回滚（发生错误则回滚）
                    cursor.close()  # 关闭游标
                    return print('传入的SQL：%s; 数据连接：%s \n 执行异常:%s ' % (self, conn, err))
                else:
                    conn.commit()  # 事务提交
                    new_nums = cursor.rowcount
                    return print('执行成功：', new_nums)
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 清除表数据
    def clear(self: str = 'null', table_name: str = 'null'):
        """
        【修改数据表的数据】\n
        :param self: str 数据库连接,如：SQL.O_conn()；
        :param table_name: str 要清除的数据表名；
        :return:  输出结果；
        """
        if null(table_name) == 'N':
            return "异常：转入的表名为空！"
        real_sql = "delete from " + table_name + ";"
        with self.cursor() as cursor:
            try:
                # 取消表的外键约束
                cursor.execute("SET FOREIGN_KEY_CHECKS=0;")
                cursor.execute(real_sql)
            except Exception as err:
                self.rollback()  # 事务回滚（发生错误则回滚）
                cursor.close()  # 关闭游标
                return print('传入的SQL：%s; \n数据连接：%s \n执行异常:%s ' % (real_sql, self, err))
            else:
                self.commit()  # 事务提交
                new_nums = cursor.rowcount
                return print('清除成功：', new_nums)

    # 关闭数据库
    def close(self):
        """
        :param self: str 数据库连接,如：SQL.O_conn()；
        :return: 无输出，直接关闭数据库连接；
        """
        if null(self) == 'N':
            return "异常：转入的数据库连接为空！"
        self.close()


# ES数据相关集合
class Es:
    """
    【ES数据相关集合】\n
    \n 目前已支持方法：
    \n 1、read_es | 链接ES数据并查询(依赖elasticsearch)
    """

    # 链接ES并查询
    def read_es(self, port, index, querys: str = '{ "query": { "match_all": {} } }'):
        """
        【连接es数据】\n
        :param self: es的host；
        :param port: 连接端口；
        :param index: 索引名；
        :param querys: 查询条件；
        :return: 查询结果
        """
        try:
            if null(self) == 'N' or null(port) == 'N' or null(index) == 'N':
                return print('入参异常：ES的host、端口、索引名 不可为空...')
            from elasticsearch import Elasticsearch, helpers
            url = {"host": self, "port": port, "timeout": 150}
            es = Elasticsearch([url])
            if es.ping():
                print("Successfully connect!")
                query = querys
                res = helpers.scan(es, index=index, scroll="20m", query=query)
                return res
            else:
                print("Failed.....")
                exit()
        except EnvironmentError as err:
            return '发生异常：%s' % err


"""新内容起"""


# Mysql数据库连接配置
class Mysql_db:
    def __init__(self, host_s='localhost', port_s=3306, user_s='root', password_s='root', db_s='new_futures'):
        """
        [数据库链接]\n
        :param host_s: 地址
        :param port_s: 端口
        :param user_s: 账号
        :param password_s: 密码
        :param db_s: 数据库
        """
        import pymysql
        try:
            self.db = pymysql.connect(host=str(host_s), port=int(port_s), user=str(user_s), password=str(password_s),
                                      db=str(db_s), charset='utf8', autocommit=True)
            self.cursor = self.db.cursor()  # 普通的游标对象，默认创建的游标对象
            self.SSCursor = self.db.cursor(cursor=pymysql.cursors.SSCursor)  # 以字典的形式返回操作结果
            self.DictCursor = self.db.cursor(cursor=pymysql.cursors.DictCursor)  # 不缓存游标，主要用于当操作需要返回大量数据的时候
            self.SSDictCursor = self.db.cursor(cursor=pymysql.cursors.SSDictCursor)  # 不缓存游标，将结果以字典的形式进行返回
        except Exception as e:
            print('发生异常：%s ' % e)

    def sel_dict(self, sql):
        """
        [以字典形式返回所有查询结果]\n
        :param sql: 传入的SQL语句
        :return: 输出查询结果(字典形式)
        """
        try:
            self.DictCursor.execute(str(sql))  # 执行SQL
            self.db.commit()  # 事务提交
            v_result = self.DictCursor.fetchall()
            return v_result
        except Exception as e:
            return print('发生异常：%s ' % e)

    def sel_sql(self, sql):
        """
        [以普通形式返回所有查询结果]\n
        :param sql: 传入的SQL语句
        :return: 输出查询结果(普通形式)
        """
        try:
            self.cursor.execute(str(sql))  # 执行SQL
            self.db.commit()  # 事务提交
            v_result = self.cursor.fetchall()
            return v_result
        except Exception as e:
            return print('发生异常：%s ' % e)

    def iud_sql(self, sql):
        """
        [以普通形式返回所有查询结果]\n
        :param sql: 传入的SQL语句
        :return: 错误和影响结果
        """
        try:
            self.cursor.execute(str(sql))  # 执行SQL
        except Exception as err:
            self.db.rollback()  # 事务回滚（发生错误则回滚）
            self.db.close()  # 关闭游标
            return print('传入的SQL：%s; 数据连接：%s \n异常信息：%s ' % (str(sql), self, err))
        else:
            self.db.commit()  # 事务提交
            update_nums = self.cursor.rowcount  # 影响数
            return print('执行成功：%s条' % update_nums)

    def close(self):
        self.db.close()

    # 列表页码及内容查询器
    def page_mus(self, table, order, ordertp='asc', page=1, listnum=10, zd='null', tp='and'):
        """
        [页码及内容查询器]\n
        :param table: 表名
        :param order: 要排序的字段名
        :param ordertp: 排序方式，默认asc
        :param page: 页码数
        :param listnum: 每页的条数
        :param zd: 条件字段及数值
        :param tp: 条件连接，默认and
        :return: page_data(总页数/上下页数) data_val(查询的页面内容) page_list(页码循环数量)
        """
        import math
        global page_num
        global vvv
        # 查询出总条数
        if zd != 'null':
            v = []  # 传入字典处理为list
            for key in zd:
                zc = str(key)
                za = " '" + str(zd[key]) + "'"
                vv = zc + za
                v.append(vv)
            tpp = ' %s ' % str(tp)
            vvv = str(tpp).join(v)  # 拼接查询条件
            list_num = "SELECT count(*) ListCount from %s where %s" % (table, vvv)
            list_nums = Mysql_db.sel_sql(self, list_num)
            self.db.commit()
        else:
            list_num = "SELECT count(*) ListCount from %s" % table
            list_nums = Mysql_db.sel_sql(self, list_num)
            self.db.commit()
        # 计算页码数
        page1 = list_nums[0][0] / listnum  # 计算页码，总条数/页条数
        pages = math.ceil(page1)  # 向前取整
        if page <= pages:
            page_num = page
        if page >= pages:
            page_num = pages
        pg1 = (page_num - 1) * listnum
        if pg1 <= 0:
            pg = 0
        else:
            pg = pg1
        # 输出页码数量list
        page_list = []
        for v in range(1, pages + 1):
            page_list.append(str(v))
        # 判断页码状态
        if pages == 1 and page == 1:
            prev_num = 1
            next_num = 1
        else:
            if page <= 1:
                prev_num = 1
                next_num = page + 1
            else:
                prev_num = page - 1
                next_num = page + 1
            if next_num == pages + 1 or page >= pages:
                next_num = page
        # 输出总页数及上下页码
        page_data = {'pages': pages, 'page': page_num, 'next_num': next_num, 'prev_num': prev_num}
        if zd != 'null':
            set_sql = "select * from {0} where 1=1 and {1} order by {2} {3} limit {4},{5};".format(table, vvv, order, ordertp, pg, listnum)
            data_val = Mysql_db.sel_dict(self, set_sql)
            self.db.commit()
        else:
            set_sql = "select * from {0} where 1=1 order by {1} {2} limit {3},{4};".format(table, order, ordertp, pg, listnum)
            data_val = Mysql_db.sel_dict(self, set_sql)
            self.db.commit()
        return page_data, data_val, page_list



# _*_ coding:utf-8 _*_
"""
常用随机数据生成类集合
\n 目前支持的有：
\n test_msg (tst_email/tst_val/tst_num/tst_one) 随机数据生成（依赖string/random）
\n @author: 'TangKaiYue'
"""

import string
import random


# 随机数据生成
class test_msg:
    """
    【随机数据生成类集合】
    \n 目前支持的有：
    \n tst_email | 邮箱随机生成（依赖string/random）
    \n tst_val | 传入文本随机生成（依赖random）
    \n tst_num | 数字随机生成（依赖random）
    \n tst_one | 随机取一个传入的值（依赖random）
    """

    # 邮箱随机生成
    def tst_email(self=['.com', '.cn', '.org', '.net'], n_min: int = 6, n_max: int = 12, z_min: int = 2, z_max: int = 5,
                  other: str = ''):
        """
        【邮箱随机生成】（依赖string/random）\n
        :param self: 邮箱后缀,默认：['.com', '.cn', '.org', '.net']；
        :param n_min: 邮箱名称最小位数,默认：6；
        :param n_max: 邮箱名称最大位数,默认：12,须大于最小位数；
        :param z_min: 邮箱后缀最小位数,默认：2；
        :param z_max: 邮箱后缀最大位数,默认：5,须大于最小位数；
        :param other: 名称里添加加其他符号；
        :return: 生成的邮箱；
        """
        try:
            # 常用邮箱后缀
            characters = string.ascii_uppercase + string.digits + str(other)  # 拼接大写+数字 的邮箱前称
            characters1 = string.ascii_lowercase + string.digits  # 拼接小写+数字 邮箱服务商名称
            # 字母数字集合
            username = ''.join((random.choice(characters) for i in range(random.randint(n_min, n_max))))
            # 6-12的随机整数用户名
            domain = ''.join((random.choice(characters1) for i in range(random.randint(z_min, z_max))))
            return username + '@' + domain + random.choice(self)
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 传入文本随机生成
    def tst_val(self: str = ('男', '女'), z_min: int = 1, z_max: int = 1):
        """
        【传入文本随机生成】（依赖random）\n
        :param self: 要生成的内容，默认为：('男', '女')；
        :param z_min: 文本最小位数,默认：1；
        :param z_max: 文本最大位数,默认：1,须大于最小位数；
        :return:  输出随机生成的结果
        """
        try:
            val = ''.join((random.choice(self) for i in range(random.randint(z_min, z_max))))
            return val
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 数字随机生成
    def tst_num(self: str = '', z_min: int = 0, z_max: int = 9, num: int = 10):
        """
        【数字随机生成】（依赖random）\n
        :param self: 数字串首个数字,默认为空；
        :param z_min: 数字最小位数,默认：0(建议0-9)；
        :param z_max: 数字最大位数,默认：9(建议0-9),须大于最小位数；
        :param num: 生成多少位数字,默认：10(有首字/大小多位时需排除算)；
        :return:  输出随机生成的结果(受最小最大多位数影响)
        """
        try:
            val = str(self) + ''.join((str(random.randint(z_min, z_max)) for i in range(num)))
            return val
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 随机取一个传入的值
    def tst_one(self: str = ('北京市', '天津市', '上海市', '重庆市', '河北省', '山西省', '辽宁省', '吉林省', '黑龙江省', '江苏省', '浙江省', '安徽省', '福建省',
                             '江西省', '山东省', '河南省', '湖北省', '湖南省', '广东省', '海南省', '四川省', '贵州省', '云南省', '陕西省', '甘肃省', '青海省',
                             '台湾省', '内蒙古自治区', '广西壮族自治区', '西藏自治区', '宁夏回族自治区', '新疆维吾尔自治区', '香港特别行政区', '澳门特别行政区')):
        """
        【随机取一个传入的值】（依赖random）\n
        :param self: 传入的文本,默认为省份数据；
        :return:  输出随机生成的结果
        """
        try:
            val = random.choice(self)
            return val
        except EnvironmentError as err:
            return '发生异常：%s' % err



# _*_ coding:utf-8 _*_
"""
tky组件 测试
\n测试文件，包含部分示例...
"""

from tky import t_sql, t_jde, t_dev, t_gui, t_msg, t_file, t_tsg


# 验证APP——GUI组件方法功能
def app_gui_test():
    appPackage = "com.xxx.app.addressbook"  # 包名
    appActivity = "com.xxx.app.addressbook.activity.WelcomeActivity"  # 包入口
    dri = t_dev.app_dev('Android', '7.1.2', 'device', appPackage, appActivity)
    print(dri)

    omr_app = "com.xxx.app.addressbook:id/Wisdom_pie_layout"  # 元素
    print(omr_app)
    t_gui.t_click('id', omr_app, dri)


# 验证PC-GUI组件方法功能
def pc_gui_test():
    url = 'https://www.baidu.com'
    dri = t_dev.pc_dev('g', url)  # 打开浏览器并进入url网页
    print(dri)
    t_gui.t_send_keys('id', 'kw', 'IP', dri)
    t_gui.t_click('xpath', '//*[@id="su"]', dri)


# 验证msg组件方法功能
def msg_test():
    wurl = 'http://localhost/service/sys/qx/send'  # 要推送的url地址
    code = '123456'  # 推送对象
    msg = '123456789123456789'  # 推送内容
    t_msg.we_msg(wurl, code, msg)


# 验证随机数生成
def test_mg():
    filename = 'test.txt'  # write txt
    # 每条信息包括：姓名、性别、年龄、籍贯、电话号码、地址、电子邮件、数学成绩、英语成绩）
    vla = '赵钱孙李周吴郑王唐余赢杜梁刘汪徐时'
    vls = '相思叶图样图森破旱田杳加盟楼主'
    vll = ('方言', '数学', '物理', '化学')
    from .t_tsg import test_msg
    with open(filename, 'w', encoding='utf-8') as fp:
        # fp.write('学号,名称,性别,年龄,籍贯,联系方式,地址,邮箱,课程,期中,期末\n')
        print('学号,名称,性别,年龄,籍贯,联系方式,地址,邮箱,课程,期中,期末\n')
        for i in range(100):
            id = test_msg.tst_num(num=6)
            name = test_msg.tst_val(vla) + test_msg.tst_val(self=vls, z_max=2)
            sex = test_msg.tst_val()
            age = test_msg.tst_num(z_min=15, z_max=30, num=1)
            native = test_msg.tst_one()
            telno = test_msg.tst_num('1', z_min=3, z_max=9)
            address = test_msg.tst_one() + test_msg.tst_val(self=vls, z_min=5, z_max=8) + '路' + test_msg.tst_num(
                num=3) + '号'
            email = test_msg.tst_email()
            student = test_msg.tst_one(vll)
            nuber1 = test_msg.tst_num(z_min=5, z_max=100, num=1)
            nuber2 = test_msg.tst_num(z_min=5, z_max=100, num=1)
            line = ','.join([id, name, sex, age, native, telno, address, email, student, nuber1, nuber2]) + '\n'
            print(line)


# 验证文件夹方法
def file_test():
    file_url = r'G:\web\test'
    furl = r'G:\web\test\test.txt'
    a = t_file.File_val.path_val(file_url, 0)
    # a = t_file.File_val.path_val(file_url)
    print(a)
    b = t_file.File_val.last_val(furl, -2)
    # b = t_file.File_val.last_val(furl )
    print(b)


# 验证文件夹方法
def excel_test():
    frl = r'G:\web\test\test.xlsx'
    # a = t_file.Excel.read_data(file_url)
    # print(a)
    b = t_file.Excel.write_data(frl, stn='Sheet1', row_n=1, dow_n=1, value='name')
    c = t_file.Excel.write_data(frl, stn='Sheet1', row_n=3, dow_n=1, value='test')
    d = t_file.Excel.write_data(frl, stn='Sheet1', row_n=1, dow_n=2, value='values')
    e = t_file.Excel.write_data(frl, stn='Sheet1', row_n=3, dow_n=2, value='test1')
    # e = t_file.Excel.write_data(file_url,stn='Sheet1',dow_n=2,value='test1')
    # e = t_file.Excel.write_data(file_url,stn='Sheet1',row_n=3,value='test1')
    print(b, c, d, e)


# 验证数据匹配方法
def msg_tst():
    b = ",'O123/one','O111/tt','t11/omm',"
    a = '123'
    c = t_jde.val_list(b, a)
    print(c)


# 验证数据SQL方法
def msg_sql():
    name = 'xxxx'
    pwd = 'xxx'
    url = 'xx.xxx.xxx.xx'
    db = 'xxxxxxx'
    cod = 3306
    p = 'xx'
    ou = 'xxx.xxx.xxx.xxx:1521/xxx'
    # 注意事项：错误1193, "Unknown system variable 'tx_isolation'"（可能是插件与数据库版本不一致导致）；1045应该是数据库账号未开通外链；
    # a = t_sql.SQL.M_conn(url, cod, name, pwd, db)
    # c = t_sql.SQL.O_conn(p, p, ou)
    # print(a)
    # osql = 'select * from xxx.xxxxxx'
    # sql = 'select * from xxx'
    # print(sql)
    # one = t_sql.SQL.set_all(sql, a)
    # one = t_sql.SQL.set_one(osql, c)
    # onel = cursor.execute(sql)
    # print(one)
    # print(onel)
    # for each in cursor.fetchone():
    #     print(each)


# 企业微信机器人消息推送
def wek_msg():
    url = ""  # 具体的企业微信配置机器人webhook地址
    text_msg = ' | 标题：企业微信消息推送[测试版]\n | 消息内容：text类型 测试信息，请忽略！\n | 操作人员：Test_msg123 \n | 操作时间：20210315'
    markdown_msg = "**加粗字体**\n" \
                   "# 我是标题一\n" \
                   "消息类型[测试]-<font color=\"warning\">markeown消息类型</font>\n " \
                   ">消息类型：<font color=\"comment\">markeown</font>\n" \
                   ">操作人员：<font color=\"comment\">test_msg123</font>\n" \
                   ">操作时间：<font color=\"comment\">20210315</font>\n" \
                   "[这是一个链接](http://work.weixin.qq.com/api/doc)"
    t_msg.wek_msg(self=url, msg_type='markdown', msg=markdown_msg)


# 字符串转换
def val_case():
    val = 'Tang tang'
    a = t_jde.val_case(val, 'u')
    b = t_jde.val_case(val, 'l')
    c = t_jde.val_case(val, 'c')
    d = t_jde.val_case(val, 't')
    print('全大写：%s \n全小写：%s \n第一个首字母大写：%s \n全首字母大写：%s' % (a, b, c, d))


# 分页并输出内容字典
def page_nums():
    from tky.t_sql import Mysql_db
    tang = Mysql_db('10.xxx.xxx.xxx', 3306, 'xxx', 'xxxx', 'xxx')  # 对应的数据链接参数
    zd = {'ltype like': '%用户%', 'ctime>': '2021-04-28'}
    # zd = {'ltype like ': '%用户%'}
    # page = Mysql_db.page_mus(tang, table='tang_pc_logs', order='ctime', ordertp='desc', page=1, listnum=15, zd=zd, tp='and')
    page = Mysql_db.page_mus(tang, table='tang_pc_logs', order='ctime', ordertp='desc', page=1, listnum=15)
    print(page[0])
    print(page[1])
    # for v in page[1]:
    #     print(v['id'])
    print(page[2])


if __name__ == '__main__':
    # app_gui_test()
    # pc_gui_test()
    # msg_test()
    # test_mg()
    # file_test()
    # excel_test()
    # msg_tst()
    # msg_sql()
    # wek_msg()
    # val_case()
    page_nums()



# _*_ coding:utf-8 _*_
"""
常用DEV类集合
\n 目前支持的方法有：
\n app_dev （移动设备连接，依赖appium）
\n pc_dev （浏览器连接，依赖selenium）
\n @author: 'TangKaiYue'
"""

from .t_jde import null


# 移动设备连接
def app_dev(osn: str = 'null', osv: str = 'null', dvn: str = 'null', package: str = 'null', activity: str = 'null',
            cod: str = '5554'):
    """
    【GUI_APP移动设备连接,端口默认为5554】\n
    :param osn: str 设备系统,如：Android；
    :param osv: str 设备系统版本,如：7.1.2；
    :param dvn: str 设备名称,如：device；
    :param package: str 设备APP包名,如：com.xxx.app.addressbook；
    :param activity: str 设备里APP包的启动入口,如：om.xxx.app.addressbook.activity.WelcomeActivity;
    :param cod: str Appium里设置的端口号，默认5554
    :return: 输出driver
    """
    try:
        if null(osn) == 'N' or null(osv) == 'N' or null(dvn) == 'N' or null(package) == 'N' or null(activity) == 'N':
            return print("异常：系统名称、系统版本、设备名称、APP包名、APP包名入口 都不能为空...")
        else:
            from appium import webdriver
            caps = {}
            caps['platformName'] = str(osn)  # 设备系统
            caps['platformVersion'] = str(osv)  # 设备系统版本
            caps['deviceName'] = str(dvn)  # 设备号
            caps['appPackage'] = str(package)  # 设备里APP包名
            caps['appActivity'] = str(activity)  # 设备里APP包的启动入口
            driver = webdriver.Remote('http://localhost:%s/wd/hub' % cod, caps)  # 启动app（端口与Appium里设置的保持一致）
            return driver
    except EnvironmentError as err:
        return '发生异常：%s' % err


# PC浏览器连接
def pc_dev(bro: str = 'g', url: str = 'null'):
    """
    【GUI_PC浏览器连接,默认最大经窗口】\n
    :param bro: str 浏览器,代码支持：g(谷歌),f(火狐)；默认g；
    :param url: str 要打开的URL链接；
    :return: 执行打开浏览器和URL的操作；
    """
    try:
        if null(bro) == 'N' or null(url) == 'N':
            return print("异常：浏览器、URL 都不能为空...")
        else:
            from selenium import webdriver
            if bro == 'g':
                driver = webdriver.Chrome()  # 驱动Chrome浏览器
                driver.maximize_window()  # 窗口最大化
                driver.get(url)  # 打开指定网址
                return driver

            elif bro == 'f':
                driver = webdriver.Firefox()  # 驱动Firefox浏览器
                driver.maximize_window()  # 窗口最大化
                driver.get(url)  # 打开指定网址
                return driver
            else:
                print('启动异常：非常抱歉，暂时只支持：g/f 谷歌/火狐, 使用其他请自行编写方法或联系作者集合...')
    except Exception as one3:
        print('\n捕捉到程序异常：%s' % one3)



# _*_ coding:utf-8 _*_
"""
常用GUI类集合
\n 目前支持的有：
\n t_click （点击事件）
\n t_clear （清除事件）
\n t_send_keys （输入事件）
\n wait_one （元素扫描）
\n @author: 'TangKaiYue'
"""

import datetime
from .t_jde import null
import selenium.webdriver.support.ui as ui  # 页面元素检查


# 扫描元素并执行-点击事件
def t_click(name: str = 'null', value: str = 'null', driver: str = 'null', sed: int = 10):
    """
    [GUI元素扫描并点击事件]\n
    :param name: str 要查询的类，如ID/NAME/CLASS/XPATH；
    :param value: str 具体的元素值；
    :param driver: str 具体的driver值；
    :param sed: int 要扫描的时长(秒),默认10S；
    :return: 执行点击操作
    """
    try:
        if null(name) == 'N' or null(value) == 'N' or null(driver) == 'N':
            return print("入参异常：查询类，具体值，driver值 不能为空...")
        print('click - type==%s ，value==%s ，time==%s' % (name, value, datetime.datetime.now()))
        if name == 'id':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_id(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_id(value).click()  # 执行操作
        if name == 'name':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_name(value).click()  # 执行操作
        if name == 'class_name':
            ui.WebDriverWait(driver, sed).until(
                lambda w: driver.find_element_by_class_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_class_name(value).click()  # 执行操作
        if name == 'xpath':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_xpath(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_xpath(value).click()  # 执行操作
    except EnvironmentError as err:
        return '发生异常：%s' % err


# 扫描元素并执行-清除事件(name请传入要查询的类，如ID/NAME/CLASS/XPATH；value请传入具体的值；)
def t_clear(name: str = 'null', value: str = 'null', driver: str = 'null', sed: int = 10):
    """
    [GUI元素扫描并点击事件]\n
    :param name: str 要查询的类，如ID/NAME/CLASS/XPATH；
    :param value: str 具体的元素值；
    :param driver: str 具体的driver值；
    :param sed: int 要扫描的时长(秒),默认10S；
    :return: 执行清除操作
    """
    try:
        if null(name) == 'N' or null(value) == 'N' or null(driver) == 'N':
            return print("入参异常：查询类，具体值，driver值 不能为空...")
        print('clear - type==%s ，value==%s ，time==%s' % (name, value, datetime.datetime.now()))
        if name == 'id':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_id(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_id(value).clear()  # 执行操作
        if name == 'name':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_name(value).clear()  # 执行操作
        if name == 'class_name':
            ui.WebDriverWait(driver, sed).until(
                lambda w: driver.find_element_by_class_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_class_name(value).clear()  # 执行操作
        if name == 'xpath':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_xpath(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_xpath(value).clear()  # 执行操作
    except EnvironmentError as err:
        return '发生异常：%s' % err


# 扫描元素并执行-输入事件(name请传入要查询的类，如ID/NAME/CLASS/XPATH；value请传入具体的值；key请传入具体的内容；)
def t_send_keys(name: str = 'null', value: str = 'null', key: str = 'null', driver: str = 'null', sed: int = 10):
    """
    [GUI元素扫描并点击事件]\n
    :param name: str 要查询的类，如ID/NAME/CLASS/XPATH；
    :param value: str 具体的元素值；
    :param key: str 要输入的内容；
    :param driver: str 具体的driver值；
    :param sed: int 要扫描的时长(秒),默认10S；
    :return: 执行输入事件
    """
    try:
        if null(name) == 'N' or null(value) == 'N' or null(key) == 'N' or null(driver) == 'N':
            return print("入参异常：查询类，具体值，driver值 不能为空...")
        print('send_keys - type==%s ，value==%s ，values==%s ，time==%s' % (name, value, key, datetime.datetime.now()))
        if str(name) == 'id':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_id(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_id(value).send_keys(key)  # 执行操作
        if str(name) == 'name':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_name(value).send_keys(key)  # 执行操作
        if str(name) == 'class_name':
            ui.WebDriverWait(driver, sed).until(
                lambda w: driver.find_element_by_class_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_class_name(value).send_keys(key)  # 执行操作
        if str(name) == 'xpath':
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_xpath(value))  # 10秒内每隔500毫秒扫描1次页面变化
            driver.find_element_by_xpath(value).send_keys(key)  # 执行操作
    except EnvironmentError as err:
        return '发生异常：%s' % err


# 设定元素扫描(单数)
def wait_one(name: str = 'null', value: str = 'null', driver: str = 'null', sed: int = 10):
    """
    [GUI元素扫描并点击事件]\n
    :param name: str 要查询的类，如ID/NAME/CLASS/XPATH；
    :param value: str 具体的元素值；
    :param driver: str 具体的driver值；
    :param sed: int 要扫描的时长(秒),默认10S；
    :return: 执行元素扫描
    """
    try:
        if null(name) == 'N' or null(value) == 'N' or null(driver) == 'N':
            return print("入参异常：查询类，具体值，driver值 不能为空...")
        print('scanning - type==%s ，value==%s ，time==%s' % (name, value, datetime.datetime.now()))
        if str(name) == str('id'):
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_id(value))  # 10秒内每隔500毫秒扫描1次页面变化
        if str(name) == str('name'):
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
        if str(name) == str('class_name'):
            ui.WebDriverWait(driver, sed).until(
                lambda w: driver.find_element_by_class_name(value))  # 10秒内每隔500毫秒扫描1次页面变化
        if str(name) == str('xpath'):
            ui.WebDriverWait(driver, sed).until(lambda w: driver.find_element_by_xpath(value))  # 10秒内每隔500毫秒扫描1次页面变化
    except EnvironmentError as err:
        return '发生异常：%s' % err



# _*_ coding:utf-8 _*_
"""
常用文件类集合
\n 目前支持方法有：
\n Jenkins_t(Job_num/file_list) （Jenkins类处理）
\n File_val(xml_msg/path_val/last_val) （文件类处理，依赖os/xml.dom）
\n Excel(read_data/write_data) （Excel处理，依赖xlrd/openpyxl）
\n @author: 'TangKaiYue'
"""

import os
from .t_jde import null


# Jenkins类集合
class Jenkins_t:
    """
    【Jenkins类集合】\n
    \n 目前已支持方法：
    \n 1、Job_num | 获取最新构建编号文件夹
    \n 2、file_list | 拼接Jenkins里构建日志的文件路径
    """

    # 获取最新的构建编号文件夹
    def Job_num(self: str = 'null'):
        """
        【获取Jenkins最新的构建编号文件夹】\n
        :param  self : str (job文件夹路径,如：C:\jenkins\Test\jobs\API_Master)；
        :return: 最新构建的文件夹编号；
        """
        try:
            if null(self) == 'N':
                return print('入参异常：job文件夹路径 不可为空...')
            else:
                f = open(str(self) + '\\nextBuildNumber', 'r')
                num = int(f.read()) - 1
                return num
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 拼接Jenkins构建日志文件的路径
    def file_list(self: str = 'null', file: str = 'null'):
        """
        【拼接Jenkins构建日志相关文件路径】\n
        :param  self : str (job文件夹路径,如：C:\jenkins\Test\jobs\API_Master)；
        :param  file : str (文件名称,如：build.xml)；
        :return: 拼接的文件路径；
        """
        try:
            if null(self) == 'N' or null(file) == 'N':
                return print('入参异常：job文件夹路径、文件名称 不可为空...')
            else:
                f = open(str(self) + '\\nextBuildNumber', 'r')
                num = int(f.read()) - 1
                path = str(self) + '\\builds\\' + str(num) + "\\" + str(file)
                return str(path)
        except EnvironmentError as err:
            return '发生异常：%s' % err


# 文件处理集合
class File_val:
    """
    【文件处理集合】\n
    \n 目前已支持方法：
    \n 1、xml_msg | 读取xml文件相应数据(依赖xml.dom)
    \n 2、path_val | 取文件夹内某个文件
    \n 3、last_val | 取某文件某行内容
    """

    # 打开xml文件，并获取对应数据
    def xml_msg(self: str = 'null', path: str = 'null'):
        """
        【打开xml文件，并获取对应数据】\n
        :param  self : str 要获取的数据组，如：['1','2']；
        :param  path : str 要打开的xml文件路径；
        :return 以list输出获取的数据；
        """
        try:
            from xml.dom import minidom  # 导入xml模块
            if null(self) == 'N' or null(path) == 'N':
                return '入参异常：请传入不为空的 self,path 值...'
            else:
                dom = minidom.parse(path)
                xml_list = []
                for e in self:
                    result = dom.getElementsByTagName(str(e))  # 获取操作结果
                    for i in range(len(result)):
                        if result[i].firstChild is None:
                            return "null"
                        else:
                            result1 = result[i].firstChild.data  # 获取节点数据
                            xml_list.append(result1)
                return xml_list
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 查询文件夹内的某个文件
    def path_val(self: str = 'null', val: int = 'null'):
        """
        【查询文件夹内的某个文件】\n
        :param  self : str 文件夹路径,如：C:\Test；
        :param  val : int  哪个文件,如最后一个：-1；不传查全部；
        :return  文件的名称；
        """
        try:
            if null(self) == 'N':
                return '入参异常：请传入不为空的 文件夹路径 值...'
            else:
                if null(val) == 'N':
                    listdir = os.listdir(self)  # 取文件夹所有
                    return str(listdir)
                else:
                    listdir = os.listdir(self)
                    val_name = listdir[int(val)]  # 取某个文件
                    return str(val_name)
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 查询某文件的某行内容
    def last_val(self: str = 'null', val: int = 'null'):
        """
        【查询某文件的某行内容】\n
        :param  self : str 文件路径；
        :param  val : int  行数,如最后一行：-1；未传取所有；
        :return  获取的最后一行内容；
        """
        try:
            if null(self) == 'N':
                return '入参异常：请传入不为空的 路径 值...'
            else:
                if null(val) == 'N':
                    with open(self, 'r', encoding='utf-8') as f:  # 打开文件
                        lines = f.readlines()  # 读取所有行
                        return str(lines)
                else:
                    with open(self, 'r', encoding='utf-8') as f:  # 打开文件
                        lines = f.readlines()  # 读取所有行
                        last_line = lines[int(val)]  # 取某一行
                        return str(last_line)
        except EnvironmentError as err:
            return '发生异常：%s' % err


# Excel处理集合
class Excel:
    """
    【Excel处理集合】\n
    \n 目前已支持方法：
    \n 1、read_data | 读取Excel表格内容(依赖xlrd)
    \n 2、write_data | 写入Excel内容(依赖openpyxl)
    """

    # 读取Excel内容
    def read_data(self: str = 'null', stn: str = "Sheet1"):
        """
        【读取Excel表格内容】\n
        :param self: str 文件路径
        :param stn: str 工作表,默认为Sheet1
        :return: 读取结果
        """
        try:
            if null(self) == 'N':
                return print('入参异常：文件路径 不可为空...')
            else:
                import xlrd
                data = xlrd.open_workbook(self)
                table = data.sheet_by_name(stn)
                # 获取总行数、总列数
                nrows = table.nrows
                ncols = table.ncols
                print('%s 工作表数据概况：共计%d行，%d列' % (stn, nrows, ncols))
                if nrows > 0:
                    # 获取第一行的内容，列表格式
                    keys = table.row_values(0)
                    list_data = []
                    # 获取每一行的内容，列表格式
                    for col in range(0, nrows):
                        values = table.row_values(col)
                        # keys，values组合转换为字典
                        val_dict = dict(zip(keys, values))
                        list_data.append(val_dict)
                    # print(list_data)
                    return list_data
                else:
                    print("表格是空数据!")
                    return None
        except EnvironmentError as err:
            return '发生异常：%s' % err

    # 写入Excel内容
    def write_data(self, stn: str = "Sheet1", row_n: int = "null", dow_n: int = "null", value: str = "null"):
        """
        【写入Excel内容】(依赖openpyxl)\n
        :param self: str 写入数据的文件路径
        :param row_n: int 所在行数,单数
        :param dow_n: int 所在列数,单数
        :param value: int 写入内容
        :param stn: str 指定工作表,默认Sheet1
        :return: 无
        """
        try:
            if not os.path.exists(self):
                return print('入参异常：文件路径异常，请检查...')
            else:
                if null(row_n) == 'N' or null(dow_n) == 'N':
                    return print('入参异常：未指定表格的行和列...')
                else:
                    from openpyxl import load_workbook
                    wb = load_workbook(self)  # 获取工作薄-写入的文件
                    ws = wb.get_sheet_by_name(stn)  # 获取工作表1-指定写入的表
                    ws.cell(row_n, dow_n, value)  # 写入内容
                    wb.save(self)
                    return print('%d行 %d列的 %s 内容写入成功！' % (row_n, dow_n, value))
        except EnvironmentError as err:
            return '发生异常：%s' % err



# _*_ coding:utf-8 _*_
"""
个人常用方法项目文件\n
 文件释义：
\n t_sql  |  SQL类集合（Oracle/ES...）
\n t_jde  |  判断类集合（判空/匹配...）
\n t_gui  |  GUI类集合（点击/清除/输入/扫描...）
\n t_msg  |  消息类集合（企业微信...）
\n t_file |  文件类集合（Jenkins/文件夹/Excel...）
\n t_dev  |  DEV类集合（APP/PC浏览器...）
\n t_tsg  |  测试数据类集合（随机数据生成...）
\n tky_test |  tky模块 测试文件（部分示例内容...）
\n @author: 'TangKaiYue'
"""

__all__ = ['t_msg', 't_file', 't_jde', 't_gui', 't_sql', 't_dev', 't_tsg', 'tky_test']



# _*_ coding:utf-8 _*_
"""
常用判断类集合
\n 目前支持方法有：
\n null (数据空判断)
\n val_list （数据匹配，自有；依赖re）
\n val_case (转换传入字符串内容)
\n Jde_o 新方法 (时间计算：change_time)
\n @author: 'TangKaiYue'
"""

# 引入时间组件
import time
from datetime import datetime, timedelta


# 数据空判断
def null(val: str = 'null'):
    """
    【数据为空判断】\n
    :param val : str 传入的内容；
    :return 为空输出'N'，不为空则输出传入内容；
    """
    try:
        if str(val) == 'null' or str(val) == '' or str(val) == [] or str(val) is None or str(val) == 'Null' or str(
                val) == 'NULL':
            return 'N'
        else:
            return str(val)
    except EnvironmentError as err:
        return '发生异常：%s' % err


# 转换传入字符串内容（大小写）
def val_case(self: str = 'null', tp: str = 'l'):
    """
    【转换传入字符串内容】\n
    :param self: 传入要转换的英文字符串（支持：转换全字符串大小写、首字母大写、各单词首字母大写）；
    :param tp: 需要输出的结果值(u:全大写/l:全小写/c:首字母大写/t:各单词首字母大写;默认输出全小写)；
    :return: 输出匹配结果；
    """
    try:
        if null(self) == 'N':
            return '异常：请传入self不为空的数据 ...'
        if tp == 'u' or tp == 'U':
            return self.upper()
        if tp == 'l' or tp == 'L':
            return self.lower()
        if tp == 'c' or tp == 'C':
            return self.capitalize()
        if tp == 't' or tp == 'T':
            return self.title()
    except EnvironmentError as err:
        return '发生异常：%s' % err


# 匹配传入的数据并返回第一个匹配结果，未传入的参数默认为空
def val_list(self: str = 'null', user: str = 'null') -> str:
    """
    【匹配传入的数据并返回第一个匹配结果】\n
    :param  self : str 用户组,前后加英文逗号.如：',用户1,用户2,用户3,'；
    :param  user : str 用户,如：用户1 (用户组内包含的部分内容)；
    :return 输出匹配结果；
    """
    try:
        import re
        if null(self) == 'N' or null(user) == 'N':
            return '异常：请传入self & user不为空的数据 ...'
        else:
            emp_name = self
            emp_code = user
        bcd = re.search('[,][^,]*' + emp_code + '[^,]*[,]', emp_name, re.M | re.I)  # 对传入的数据进行匹配处理
        if bcd is None:  # 判断匹配的结果
            return '未匹配到对应结果，None'
        val = bcd.group()
        new_name = val[1:-1]  # 截取匹配的结果内容
        return new_name
    except EnvironmentError as err:
        return '发生异常：%s' % err


# 新方法类
class Jde_o:
    # 时间计算
    def change_time(days=0, hours=0, minutes=0):
        """
        [时间计算]\n
        :param days: 要计算的天
        :param hours: 要计算的时
        :param minutes: 要计算的分
        :return: 计算后的结果
        """
        # 定义现在时间
        new_time = time.strftime("%Y-%m-%d %H:%M:%S")
        if type(new_time) == str:
            time_str = datetime.strptime(new_time, '%Y-%m-%d %H:%M:%S')
            ret = time_str + timedelta(days=days, hours=hours, minutes=minutes)
            return ret



# _*_ coding:utf-8 _*_
from setuptools import find_packages, setup

setup(
    name='tky',  # 模块名称
    version='1.2.0',  # 模块版本号
    author='Tang kai yue',  # 作者名
    author_email='tky_1314@163.com',  # 作者邮箱
    maintainer='Tang kai yue',  # 维护者
    maintainer_email='tky_1314@163.com',  # 维护者邮箱
    description='tang kai yue 个人使用python方法集合',  # 模块描述
    long_description_content_type='text/markdown',  # 文档格式
    long_description=open('README.md', 'r', encoding='UTF-8').read(),  # 这里是文档内容, 读取readme文件
    url="https://gitee.com/tangkaiyue/tky",
    packages=find_packages(),
    # install_requires=['xml', 'os', 'urllib'],  # 需要依赖的包
    classifiers=[  # 这里我们指定证书, python版本和系统类型
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"],
    python_requires='>=3',  # 这里指定python版本号必须大于等于3才可以安装
)



# _*_ coding:utf-8 _*_
"""
常用数据类函数集合
\n 目前支持方法有：
\n we_msg （微信自有接口消息推送，依赖Json/urllib.request）
\n wek_msg （推送企业微信机器人消息，依赖Json/urllib.request）
\n @author: 'TangKaiYue'
"""

from .t_jde import null


# 企业微信推送（自有接口）
def we_msg(self: str = 'null', emp_list: str = 'null', msg: str = 'null'):
    """
    【从自有消息接口推送企业微信消息】\n
    :param  self : str 推送的地址；
    :param  emp_list : str 推送的对象(如：“用户1,用户2”)；
    :param  msg : str 消息体；
    :return  输出结果；
    """
    try:
        if null(self) == 'N' or null(msg) == 'N' or null(emp_list) == 'N':
            return '入参异常：请传入 self、msg、emp_list 不为空的数据 ...'
        else:
            import json
            import urllib.request
            # 企业微信推送接口的内容
            con = {"empCode": emp_list, "message": msg}
            print(con)
            new_msg = json.dumps(con).encode('utf-8')
            req = urllib.request.Request(self, new_msg)
            req.add_header('Content-Type', 'application/json')
            we_mse = urllib.request.urlopen(req)
            return we_mse
    except EnvironmentError as err:
        return '发生异常：%s' % err


# 企业微信推送（外部服务）
def wek_msg(self: str = 'null', msg_type: str = 'text', msg: str = 'null'):
    """
    【推送企业微信机器人消息】\n
    注：每个机器人发送的消息不能超过20条/分钟。\n
    :param  self : str 企业微信配置机器人webhook地址；
    :param  msg_type : str 消息类型(text/markdown/image/file),暂时只支持text/markdown；
    :param  msg : str 推送的消息内容；
    :return  输出结果；
    """
    try:
        if null(self) == 'N' or null(msg) == 'N':
            return '入参异常：请传入 self、msg 不为空的数据 ...'
        import json
        import urllib.request
        if msg_type == 'text' or msg_type == 'TEXT' or msg_type == 'Text':
            # 企业微信text推送接口的内容
            con = {"msgtype": 'text', "text": {"content": msg}}
            print(con)
            new_msg = json.dumps(con).encode('utf-8')
            req = urllib.request.Request(self, new_msg)
            req.add_header('Content-Type', 'application/json')
            we_mse = urllib.request.urlopen(req)
            return we_mse
        if msg_type == 'markdown' or msg_type == 'MARKDOWN' or msg_type == 'Markdown':
            # 企业微信text推送接口的内容
            con = {"msgtype": 'markdown', "markdown": {"content": msg}}
            print(con)
            new_msg = json.dumps(con).encode('utf-8')
            req = urllib.request.Request(self, new_msg)
            req.add_header('Content-Type', 'application/json')
            we_mse = urllib.request.urlopen(req)
            return we_mse
    except EnvironmentError as err:
        return '发生异常：%s' % err
